import platform

from coalib.bearlib.abstractions.CorrectionBasedBear import CorrectionBasedBear
from coalib.bearlib.spacing.SpacingHelper import SpacingHelper


class IndentBear(CorrectionBasedBear):
    executable = "indent" if platform.system() != "Darwin" else "gindent"
    RESULT_MESSAGE = "Indentation can be improved."

    def run(self,
            filename,
            file,
            max_line_length: int=80,
            use_spaces: bool=True,
            tab_width: int=SpacingHelper.DEFAULT_TAB_WIDTH,
            indent_cli_options: str=''):
        """
        This bear checks and corrects spacing and indentation via the well
        known Indent utility. It is designed to work with the C programming
        language but may work reasonably with syntactically similar languages.

        :param max_line_length:    Maximum number of characters for a line.
        :param use_spaces:         True if spaces are to be used, else tabs.
        :param tab_width:          Number of spaces per indent level.
        :param indent_cli_options: Any command line options the indent binary
                                   understands. They will be simply passed
                                   through.
        """
        options = "--no-tabs" if use_spaces else "--use-tabs"
        options += (" --line-length {0} --indent-level {1} --tab-size {1} "
                    "{2}".format(max_line_length,
                                 tab_width,
                                 indent_cli_options))
        for result in self.retrieve_results(filename,
                                            file,
                                            cli_options=options):
            yield result



from coalib.misc.Decorators import generate_ordering, generate_repr
from coalib.parsing.StringProcessing import Match


@generate_repr("begin", "inside", "end")
@generate_ordering("begin", "inside", "end")
class InBetweenMatch:
    """
    Holds information about a match enclosed by two matches.
    """

    def __init__(self, begin, inside, end):
        """
        Instantiates a new InBetweenMatch.

        :param begin:  The `Match` of the start pattern.
        :param inside: The `Match` between start and end.
        :param end:    The `Match` of the end pattern.
        """
        if begin > inside or inside > end:
            raise ValueError("The inside match must be enclosed by the begin "
                             "and end match.")

        self._begin = begin
        self._inside = inside
        self._end = end

    @classmethod
    def from_values(cls, begin, begin_pos, inside, inside_pos, end, end_pos):
        """
        Instantiates a new InBetweenMatch from Match values.

        This function allows to bypass the usage of Match object instantation:

        >>> InBetweenMatch(Match("A", 0), Match("B", 1), Match("B", 2))

        can be simplified to:

        >>> InBetweenMatch.from_values("A", 0, "B", 1, "C", 2)

        :param begin:      The matched string from start pattern.
        :param begin_pos:  The position of the matched begin string.
        :param inside:     The matched string from inside/in-between pattern.
        :param inside_pos: The position of the matched inside/in-between
                           string.
        :param end:        The matched string from end pattern.
        :param end_pos:    The position of the matched end string.
        :returns:          An InBetweenMatch from the given values.
        """
        return cls(Match(begin, begin_pos),
                   Match(inside, inside_pos),
                   Match(end, end_pos))

    @property
    def begin(self):
        return self._begin

    @property
    def inside(self):
        return self._inside

    @property
    def end(self):
        return self._end



import re

from coalib.bearlib.abstractions.Lint import Lint
from coalib.bears.LocalBear import LocalBear
from coalib.results.RESULT_SEVERITY import RESULT_SEVERITY
from coalib.misc.Shell import escape_path_argument


class JSHintBear(LocalBear, Lint):
    executable = 'jshint'
    arguments = '--verbose'
    output_regex = re.compile(
        r'.+?: line (?P<line>\d+), col (?P<col>\d+), '
        r'(?P<message>.+) \((?P<severity>\S)\d+\)')
    severity_map = {
        "E": RESULT_SEVERITY.MAJOR,
        "W": RESULT_SEVERITY.NORMAL,
        "I": RESULT_SEVERITY.NORMAL}

    def run(self,
            filename,
            file,
            jshint_config: str=""):
        '''
        Checks the code with jshint. This will run jshint over each file
        separately.

        :param jshint_config: The location of the jshintrc config file.
        '''
        if jshint_config:
            self.arguments += (" --config "
                               + escape_path_argument(jshint_config))

        return self.lint(filename)



from coalib.bearlib.abstractions.CorrectionBasedBear import CorrectionBasedBear


class GofmtBear(CorrectionBasedBear):
    executable = 'gofmt'
    RESULT_MESSAGE = "Formatting can be improved."

    def run(self, filename, file):
        """
        Proposes corrections of Go code using gofmt.
        """
        for result in self.retrieve_results(filename, file, cli_options=''):
            yield result



import re

from coalib.bearlib.abstractions.Lint import Lint
from coalib.bears.LocalBear import LocalBear
from coalib.results.RESULT_SEVERITY import RESULT_SEVERITY


class RubyLintBear(LocalBear, Lint):
    executable = 'ruby'
    arguments = '-wc'
    output_regex = re.compile(
        r'(?P<file_name>.+?):(?P<line>\d+): (?P<message>'
        r'.*?(?P<severity>error|warning)[,:] [^\r\n]+)\r?\n'
        r'(?:^[^\r\n]+\r?\n^(?P<col>.*?)\^)?')
    use_stderr = True
    severity_map = {
        "warning": RESULT_SEVERITY.NORMAL,
        "error": RESULT_SEVERITY.MAJOR}

    def run(self, filename, file):
        '''
        Checks the code with `ruby -wc` on each file separately.
        '''
        return self.lint(filename)



"""
This directory holds language definitions.

Language definitions hold expressions that help defining specific syntax
elements for a programming language.

Currently defined keys are:

  names
  extensions
  comment delimiter
  multiline comment delimiters
  string delimiters
  multiline string delimiters
"""
